/* $LAAS: viamlib.h 2011/03/08 15:41:00 mallet $
 *
 * Copyright (c) 2007-2011 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Mon Jan 22 2007
 */
#ifndef H_VIAMLIB
#define H_VIAMLIB

#include <sys/time.h>
#include <stdlib.h>

#include "viam/viamtypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/* --- Library generic routines -------------------------------------- */

typedef struct viam_handle *	viam_handle_t;

viam_handle_t		viam_init(void);
void			viam_release(viam_handle_t vh);
viam_errorcode_t	viam_errno(viam_handle_t vh);


/* --- Images -------------------------------------------------------------- */

#if 0
typedef viam_image_str const *	viam_image_obj;
#endif
typedef struct viam_image *	viam_image_ref;

typedef int (*viam_lockfn)(void *data);


/* --- Image sources ------------------------------------------------- */

typedef struct viam_camera *	viam_camera_t;

viam_camera_t	viam_camera_create(viam_handle_t vh, const char *uid,
			const char *name);


/* --- Bank and camera configuration --------------------------------- */

typedef struct viam_bank *	viam_bank_t;

viam_bank_t	viam_bank_create(viam_handle_t vh, const char *name);
void		viam_bank_delete(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_cameraadd(viam_handle_t vh, viam_bank_t bank,
			const viam_camera_t camera, const char *name);
const char *	viam_bank_name(viam_bank_t bank);
int		viam_bank_nimages(viam_bank_t bank);
int		viam_bank_ncameras(viam_bank_t bank);
viam_image_ref	viam_bank_image(viam_handle_t vh, viam_bank_t bank, int i);
viam_camera_t	viam_bank_camera(viam_handle_t vh, viam_bank_t bank, int i);
void *		viam_bank_usrdata(viam_bank_t bank);
void		viam_bank_setusrdata(viam_bank_t bank, void *usrdata);

void		viam_camera_delete(viam_handle_t vh, viam_camera_t camera);
const char *	viam_camera_name(viam_camera_t cam);
const char *	viam_camera_uid(viam_camera_t c);
const viam_hwmode_t *
		viam_camera_hwmode(viam_handle_t vh, viam_camera_t camera);
int		viam_camera_sethwmode(viam_handle_t vh, viam_camera_t camera,
			const viam_hwmode_t *hwmode);
void *		viam_camera_usrdata(viam_camera_t camera);
void		viam_camera_setusrdata(viam_camera_t camera, void *usrdata);

const char *	viam_image_name(viam_image_ref image);
viam_camera_t	viam_image_camera(const viam_image_ref image);
int		viam_image_geometry(viam_handle_t vh, const viam_image_ref image,
			viam_geometry_t *geom);

viam_bank_t	viam_bank_getbyname(const viam_handle_t vh, const char *name);
viam_camera_t	viam_camera_getbyuid(const viam_handle_t vh, const char *uid);
viam_camera_t	viam_camera_getbyname(const viam_handle_t vh, const char *name);
viam_image_ref	viam_image_getbyname(const viam_handle_t vh, const char *name);


/* --- Hardware control ---------------------------------------------- */

int		viam_hardware_load(viam_handle_t vh, const char *driver);
int		viam_hardware_unload(viam_handle_t vh, const char *driver);

int		viam_hardware_list(viam_handle_t vh);
int		viam_hardware_attach(viam_handle_t vh);
void		viam_hardware_detach(viam_handle_t vh);

int		viam_bank_configure(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_unconfigure(viam_handle_t vh, viam_bank_t bank);
int		viam_bank_isconfigured(viam_bank_t bank);

int		viam_queryhwmodes(viam_handle_t vh, viam_camera_t camera,
			int offset, viam_hwmode_t *supported, size_t n);
int		viam_hwmode_size(viam_handle_t vh, const viam_hwmode_t *hwmode,
			int *width, int *height, int *depth, int *channels);

int		viam_datatransmit(viam_handle_t vh, viam_bank_t bank,
			viam_onoff_t on);


/* --- Image filters ------------------------------------------------- */

typedef struct viam_filter *		viam_filter_t;

int		viam_queryfeature(viam_handle_t vh, viam_camera_t camera,
			viam_filtertype_t filter, unsigned int *methods,
			unsigned int *automodes);

viam_filter_t	viam_filter_format_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type, viam_filterformat_t format);
int		viam_filter_format_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,
			viam_filterformat_t format);
viam_filter_t	viam_filter_geometry_create(viam_handle_t vh,
			const char *name, viam_filtertype_t type,
			double left, double top, double right, double bottom,
			double towidth, double toheight);
int		viam_filter_geometry_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,
			double left, double top, double right, double bottom,
			double towidth, double toheight);
viam_filter_t	viam_filter_luminance_create(viam_handle_t vh,
			const char *name, viam_filtertype_t type,
			viam_valuekind_t vkind, double parameter);
int		viam_filter_luminance_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,  viam_valuekind_t vkind,
			double parameter);
viam_filter_t	viam_filter_color_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type,  viam_valuekind_t vkind,
			double parameter);
int		viam_filter_color_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,  viam_valuekind_t vkind,
			double parameter);
viam_filter_t	viam_filter_improc_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type,  viam_valuekind_t vkind,
			double parameter);
int		viam_filter_improc_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,  viam_valuekind_t vkind,
			double parameter);
viam_filter_t	viam_filter_misc_create(viam_handle_t vh, const char *name,
			viam_filtertype_t type,  viam_valuekind_t vkind,
			double parameter);
int		viam_filter_misc_update(viam_handle_t vh,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t aumode,  viam_valuekind_t vkind,
			double parameter);

int		viam_filter_push(viam_handle_t vh, viam_image_ref camera,
			viam_filter_t filter, viam_filtermethod_t method,
			viam_filterautomode_t automode);

viam_filter_t	viam_filter_getbyname(viam_handle_t vh, const char *name);

const char *		viam_filter_name(const viam_filter_t filter);
viam_filtertype_t	viam_filter_type(const viam_filter_t filter);
viam_filtermethod_t	viam_filter_method(const viam_filter_t filter);
viam_filterautomode_t	viam_filter_autoarg(const viam_filter_t filter);


/* --- Camera calibration -------------------------------------------- */

int		viam_image_calibration(viam_handle_t vh, viam_image_ref image,
			viam_cameracalibration_t *cal, const viam_geometry_t *geom);
int		viam_bank_calibration(viam_handle_t vh, viam_bank_t bank,
			viam_bankcalibration_t *cal);

int		viam_calibration_save(viam_handle_t vh, viam_bank_t bank,
			const char *file);
int		viam_calibration_load(viam_handle_t vh, viam_bank_t bank,
			const char *file);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* H_VIAMLIB */
